-- GUI for Macro Broker, by Tuhljin
-- Portions of this code are based on Blizzard_MacroUI.

MacroBrokerGUI = {}
local MacroBrokerGUI = MacroBrokerGUI
local MacroBroker = MacroBroker

local ldb = LibStub:GetLibrary("LibDataBroker-1.1")

local L = MACROBROKER_STRINGS

local MAX_CHARACTERS_PER_LINE = 255
local MAX_CHARACTERS = 1023

local MAX_ACCOUNT_MACROS = 36;
local MAX_CHARACTER_MACROS = 36;
local NUM_MACROS_PER_ROW = 6;
local NUM_MACRO_ICONS_SHOWN = 20;
local NUM_ICONS_PER_ROW = 5;
local NUM_ICON_ROWS = 4;

MACROBROKERGUI_ICON_ROW_HEIGHT = 36;

local QuestionMarkIcon = "Interface\\Icons\\INV_Misc_QuestionMark"

local MACROS = {}
local numAccountMacros, numCharacterMacros = 0, 0

local function sort_caseinsensitive(a, b)
  local a2, b2 = strlower(a), strlower(b)
  if (a2 == b2) then
    return a < b
  else
    return a2 < b2
  end
end

local temptab = {}
local function UpdateMacroList()
	wipe(MACROS)
	numAccountMacros, numCharacterMacros = 0, 0
	local tab, icon, body, perChar
	for obj, name in MacroBroker.IterateObjects() do
		icon, body, perChar = MacroBroker.GetMacroInfo(name)
		if (body) then
			if (perChar) then
				numCharacterMacros = numCharacterMacros + 1
				temptab[numCharacterMacros] = name
			else
				numAccountMacros = numAccountMacros + 1
				MACROS[numAccountMacros] = name
			end
		end
	end
	sort(MACROS, sort_caseinsensitive)
	sort(temptab, sort_caseinsensitive)
	for i,name in ipairs(temptab) do
		MACROS[MAX_ACCOUNT_MACROS + i] = name
	end
	wipe(temptab)
end

local function ClearEditBoxFocus()
	MacroBrokerGUIFrameText:ClearFocus()
	MacroBrokerGUI_LabelEditBox:ClearFocus()
	MacroBrokerGUI_TextEditBox:ClearFocus()
end


UIPanelWindows["MacroBrokerGUIFrame"] = { area = "left", pushable = 5, whileDead = 1 };

function MacroBrokerGUI.Show()
	ShowUIPanel(MacroBrokerGUIFrame);
end

function MacroBrokerGUI.OnLoad(self)
	MacroBrokerGUIFrameTitle:SetText( GetAddOnMetadata("MacroBroker", "Title").." v"..GetAddOnMetadata("MacroBroker", "Version") )
	MacroBrokerGUIFrameEditButton:SetText(L.BUTTON_CHANGE_NAME_ICON_TYPE)
	MacroBrokerGUIFrameLabelLabel:SetText(L.EDITBOX_LABEL)
	MacroBrokerGUIFrameLabelText:SetText(L.EDITBOX_TEXT)
	tinsert(MacroBrokerGUIFrame.EditBoxList, MacroBrokerGUIFrameText)

	UpdateMacroList()
	MacroBrokerGUI.SetAccountMacros();
	PanelTemplates_SetNumTabs(MacroBrokerGUIFrame, 2);
	PanelTemplates_SetTab(MacroBrokerGUIFrame, 1);
end

function MacroBrokerGUI.OnShow(self)
	MacroBrokerGUI.Update();
	PlaySound("igCharacterInfoOpen");
end

function MacroBrokerGUI.OnHide(self)
	MacroBrokerGUI_IconPopupFrame:Hide();
	MacroBrokerGUI.SaveMacro();
	PlaySound("igCharacterInfoClose");
end

function MacroBrokerGUI.SetAccountMacros()
	MacroBrokerGUIFrame.macroBase = 0;
	MacroBrokerGUIFrame.macroMax = MAX_ACCOUNT_MACROS;
	if ( numAccountMacros > 0 ) then
		MacroBrokerGUI.SelectMacro(MacroBrokerGUIFrame.macroBase + 1);
	else
		MacroBrokerGUI.SelectMacro(nil);
	end
end

function MacroBrokerGUI.SetCharacterMacros()
	MacroBrokerGUIFrame.macroBase = MAX_ACCOUNT_MACROS;
	MacroBrokerGUIFrame.macroMax = MAX_CHARACTER_MACROS;
	if ( numCharacterMacros > 0 ) then
		MacroBrokerGUI.SelectMacro(MacroBrokerGUIFrame.macroBase + 1);
	else
		MacroBrokerGUI.SelectMacro(nil);
	end
end

function MacroBrokerGUI.Update()
	UpdateMacroList()

	local numMacros;
	local macroButtonName, macroButton, macroIcon, macroName;
	local name, texture, body;
	local selectedName, selectedBody;

	if ( MacroBrokerGUIFrame.macroBase == 0 ) then
		numMacros = numAccountMacros;
	else
		numMacros = numCharacterMacros;
	end

	-- Macro List
	local maxMacroButtons = max(MAX_ACCOUNT_MACROS, MAX_CHARACTER_MACROS);
	MacroBrokerGUI_IconPopupFrame.selectedIconTexture = nil
	for i=1, maxMacroButtons do
		macroButtonName = "MacroBrokerGUIButton"..i;
		macroButton = _G[macroButtonName];
		macroIcon = _G[macroButtonName.."Icon"];
		macroName = _G[macroButtonName.."Name"];
		if ( i <= MacroBrokerGUIFrame.macroMax ) then
			if ( i <= numMacros ) then
				name = MACROS[MacroBrokerGUIFrame.macroBase + i]
				texture, body = MacroBroker.GetMacroInfo(name);
				macroIcon:SetTexture(texture);
				macroName:SetText(name);
				if (not MacroBroker.IsPluginSaved(name)) then
					macroName:SetTextColor(1, 0.1, 0.1)
				elseif (MacroBroker.GetRealName(name) ~= name) then
					macroName:SetTextColor(1, 1, 0.1)
				else
					macroName:SetTextColor(1, 1, 1)
				end
				macroButton:Enable();
				-- Highlight Selected Macro
				if ( MacroBrokerGUIFrame.selectedMacro and (i == (MacroBrokerGUIFrame.selectedMacro - MacroBrokerGUIFrame.macroBase)) ) then
					macroButton:SetChecked(1);
					MacroBrokerGUIFrameSelectedMacroName:SetText(name);
					MacroBrokerGUIFrameText:SetText(body);
					MacroBrokerGUIFrameSelectedMacroButton:SetID(i);
					MacroBrokerGUIFrameSelectedMacroButtonIcon:SetTexture(texture);
					MacroBrokerGUI_IconPopupFrame.selectedIconTexture = texture;
				else
					macroButton:SetChecked(0);
				end
			else
				macroButton:SetChecked(0);
				macroIcon:SetTexture("");
				macroName:SetText("");
				macroButton:Disable();
			end
			macroButton:Show();
		else
			macroButton:Hide();
		end
	end

	-- Macro Details
	if ( MacroBrokerGUIFrame.selectedMacro ~= nil ) then
		MacroBrokerGUI.ShowDetails();
		MacroBrokerGUIFrameDeleteButton:Enable();
		local obj = ldb:GetDataObjectByName( MACROS[MacroBrokerGUIFrame.selectedMacro] )
		if (MacroBroker.IsPluginSaved(obj)) then
			MacroBrokerGUIFrameDeleteButton:SetText(DELETE)
			MacroBrokerGUIFrameEnterMacroText:SetText(ENTER_MACRO_LABEL)
		else
			MacroBrokerGUIFrameDeleteButton:SetText(L.BUTTON_UNDELETE)
			MacroBrokerGUIFrameEnterMacroText:SetText(ENTER_MACRO_LABEL.."  |cffff0000"..L.LABEL_DELETE_MARKED)
		end
		MacroBrokerGUI_LabelEditBox:SetText(obj.label or "")
		MacroBrokerGUI_TextEditBox:SetText(obj.text or "")
	else
		MacroBrokerGUI.HideDetails();
		MacroBrokerGUIFrameDeleteButton:Disable();
		MacroBrokerGUIFrameDeleteButton:SetText(DELETE)
	end

	--Update New Button
	if ( numMacros < MacroBrokerGUIFrame.macroMax ) then
		MacroBrokerGUIFrameNewButton:Enable();
	else
		MacroBrokerGUIFrameNewButton:Disable();
	end

	-- Disable Buttons
	if ( MacroBrokerGUI_IconPopupFrame:IsShown() ) then
		MacroBrokerGUIFrameEditButton:Disable();
		MacroBrokerGUIFrameDeleteButton:Disable();
	else
		MacroBrokerGUIFrameEditButton:Enable();
		MacroBrokerGUIFrameDeleteButton:Enable();
	end

	if ( not MacroBrokerGUIFrame.selectedMacro ) then
		MacroBrokerGUIFrameDeleteButton:Disable();
	end
end

function MacroBrokerGUI.AddMacroLine(line)
	if ( MacroBrokerGUIFrameText:IsVisible() ) then
		MacroBrokerGUIFrameText:SetText(MacroBrokerGUIFrameText:GetText()..line);
	end
end

function MacroBrokerGUI.MacroButton_OnClick(self, button)
	MacroBrokerGUI.SaveMacro();
	MacroBrokerGUI.SelectMacro(MacroBrokerGUIFrame.macroBase + self:GetID());
	MacroBrokerGUI.Update();
	MacroBrokerGUI_IconPopupFrame:Hide();
	ClearEditBoxFocus()
end

function MacroBrokerGUI.SelectMacro(id)
	MacroBrokerGUIFrame.selectedMacro = id;
end

function MacroBrokerGUI.NewButton_OnClick(self, button)
	MacroBrokerGUI.SaveMacro();
	MacroBrokerGUI_IconPopupFrame.mode = "new";
	MacroBrokerGUI_IconPopupFrame:Show();
end

function MacroBrokerGUI.EditButton_OnClick(self, button)
	MacroBrokerGUI.SaveMacro();
	MacroBrokerGUI_IconPopupFrame.mode = "edit";
	local name = MACROS[MacroBrokerGUIFrame.selectedMacro]
	MacroBrokerGUIFrameSelectedMacroName:SetText( MacroBroker.GetRealName(name) )
	MacroBrokerGUI_IconPopupFrame:Show();
end

function MacroBrokerGUI.HideDetails(extra)
	MacroBrokerGUIFrameLabelLabel:Hide()
	MacroBrokerGUI_LabelEditBox:Hide()
	MacroBrokerGUIFrameLabelText:Hide()
	MacroBrokerGUI_TextEditBox:Hide()
	if (extra) then  return;  end

	MacroBrokerGUIFrameEditButton:Hide();
	MacroBrokerGUIFrameCharLimitText:Hide();
	MacroBrokerGUIFrameText:Hide();
	MacroBrokerGUIFrameSelectedMacroName:Hide();
	MacroBrokerGUIFrameSelectedMacroBackground:Hide();
	MacroBrokerGUIFrameSelectedMacroButton:Hide();
end

function MacroBrokerGUI.ShowDetails()
	MacroBrokerGUIFrameLabelLabel:Show()
	MacroBrokerGUI_LabelEditBox:Show()
	MacroBrokerGUIFrameLabelText:Show()
	MacroBrokerGUI_TextEditBox:Show()

	MacroBrokerGUIFrameEditButton:Show();
	MacroBrokerGUIFrameCharLimitText:Show();
	MacroBrokerGUIFrameEnterMacroText:Show();
	MacroBrokerGUIFrameText:Show();
	MacroBrokerGUIFrameSelectedMacroName:Show();
	MacroBrokerGUIFrameSelectedMacroBackground:Show();
	MacroBrokerGUIFrameSelectedMacroButton:Show();
end

function MacroBrokerGUI.MacroButtonContainer_OnLoad(self)
	local button;
	local maxMacroButtons = max(MAX_ACCOUNT_MACROS, MAX_CHARACTER_MACROS);
	for i=1, maxMacroButtons do
		button = CreateFrame("CheckButton", "MacroBrokerGUIButton"..i, self, "MacroBrokerGUI_MacroButtonTemplate");
		button:SetID(i);
		if ( i == 1 ) then
			button:SetPoint("TOPLEFT", self, "TOPLEFT", 6, -6);
		elseif ( mod(i, NUM_MACROS_PER_ROW) == 1 ) then
			button:SetPoint("TOP", "MacroBrokerGUIButton"..(i-NUM_MACROS_PER_ROW), "BOTTOM", 0, -10);
		else
			button:SetPoint("LEFT", "MacroBrokerGUIButton"..(i-1), "RIGHT", 13, 0);
		end
	end
end

function MacroBrokerGUI.IconPopupFrame_OnShow(self)
	if (not MacroBrokerGUI_IconPopupButton1) then  -- Create icon buttons dynamically:
		local button
		for i=1, NUM_MACRO_ICONS_SHOWN do
			button = CreateFrame("CheckButton", "MacroBrokerGUI_IconPopupButton"..i, self, "MacroBrokerGUI_IconPopupButtonTemplate");
			button:SetID(i);
			if ( i == 1 ) then
				button:SetPoint("TOPLEFT", self, "TOPLEFT", 24, -85);
			elseif ( mod(i, NUM_ICONS_PER_ROW) == 1 ) then
				button:SetPoint("TOPLEFT", "MacroBrokerGUI_IconPopupButton"..(i-NUM_ICONS_PER_ROW), "BOTTOMLEFT", 0, -8);
			else
				button:SetPoint("LEFT", "MacroBrokerGUI_IconPopupButton"..(i-1), "RIGHT", 10, 0);
			end
		end
	end
	
	MacroBrokerGUI_IconPopupFrame.selectedIcon = nil
	PlaySound("igCharacterInfoOpen");
	MacroBrokerGUI.IconPopupFrame_Update(self);
	MacroBrokerGUI.IconPopupOkayButton_Update();

	if ( self.mode == "new" ) then
		MacroBrokerGUIFrameText:Hide();
		MacroBrokerGUI.IconPopupButton_SelectTexture(1)
		MacroBrokerGUI_IconPopupEditBox:SetFocus();
		MacroBrokerGUI.HideDetails(true)
	end

	-- Disable Buttons
	MacroBrokerGUIFrameEditButton:Disable();
	MacroBrokerGUIFrameDeleteButton:Disable();
	MacroBrokerGUIFrameNewButton:Disable();
	MacroBrokerGUIFrameTab1:Disable();
	MacroBrokerGUIFrameTab2:Disable();
end

function MacroBrokerGUI.IconPopupFrame_OnHide(self)
	if ( self.mode == "new" ) then
		MacroBrokerGUIFrameText:Show();
		--MacroBrokerGUIFrameText:SetFocus();
	end
	
	-- Enable Buttons
	MacroBrokerGUIFrameEditButton:Enable();
	MacroBrokerGUIFrameDeleteButton:Enable();
	local numMacros;
	if ( MacroBrokerGUIFrame.macroBase == 0 ) then
		numMacros = numAccountMacros;
	else
		numMacros = numCharacterMacros;
	end
	if ( numMacros < MacroBrokerGUIFrame.macroMax ) then
		MacroBrokerGUIFrameNewButton:Enable();
	end
	-- Enable tabs
	PanelTemplates_UpdateTabs(MacroBrokerGUIFrame);
end

function MacroBrokerGUI.IconPopupFrame_Update(self)
	self = self or MacroBrokerGUI_IconPopupFrame;
	local numMacroIcons = GetNumMacroIcons();
	local MacroBrokerGUI_IconPopupIcon, MacroBrokerGUI_IconPopupButton;
	local MacroBrokerGUI_IconPopupOffset = FauxScrollFrame_GetOffset(MacroBrokerGUI_IconPopupScrollFrame);
	local index;
	local name = MACROS[MacroBrokerGUIFrame.selectedMacro]

	-- Determine whether we're creating a new macro or editing an existing one
	if ( self.mode == "new" ) then
		MacroBrokerGUI_IconPopupEditBox:SetText("");
		MacroBrokerGUI_IconPopupFrameNameLabel:SetText(L.LABEL_MACRONAME_NEW)
		MacroBrokerGUI_IconPopupEditBox:SetAutoFocus(true)
		MacroBrokerGUI_IconPopupLauncherCheckBox:SetChecked(1)
		MacroBrokerGUI_IconPopupLauncherCheckBox.tip2 = nil
	elseif ( self.mode == "edit" ) then
		MacroBrokerGUI_IconPopupEditBox:SetText( MacroBroker.GetRealName(name) );
		MacroBrokerGUI_IconPopupFrameNameLabel:SetText(L.LABEL_MACRONAME_EDIT)
		MacroBrokerGUI_IconPopupEditBox:SetAutoFocus(false)
		MacroBrokerGUI_IconPopupEditBox:ClearFocus()
		ClearEditBoxFocus()
		local obj = ldb:GetDataObjectByName(name)
		if (obj.type == "launcher") then
			MacroBrokerGUI_IconPopupLauncherCheckBox:SetChecked(1)
		else
			MacroBrokerGUI_IconPopupLauncherCheckBox:SetChecked(0)
		end
		MacroBrokerGUI_IconPopupLauncherCheckBox.tip2 = L.CHECKBOX_LAUNCHER_TIP2
	end

	-- Icon list
	local selectedIcon, selectedIconTexture = MacroBrokerGUI_IconPopupFrame.selectedIcon, MacroBrokerGUI_IconPopupFrame.selectedIconTexture
	if (selectedIconTexture) then
		local _, _, _, isDynamicIcon = MacroBroker.GetMacroInfo(name)
		if (isDynamicIcon) then  selectedIconTexture = QuestionMarkIcon;  end
	end
	for i=1, NUM_MACRO_ICONS_SHOWN do
		MacroBrokerGUI_IconPopupIcon = _G["MacroBrokerGUI_IconPopupButton"..i.."Icon"];
		MacroBrokerGUI_IconPopupButton = _G["MacroBrokerGUI_IconPopupButton"..i];
		index = (MacroBrokerGUI_IconPopupOffset * NUM_ICONS_PER_ROW) + i;
		texture = GetMacroIconInfo(index);
		if ( index <= numMacroIcons ) then
			MacroBrokerGUI_IconPopupIcon:SetTexture(texture);
			MacroBrokerGUI_IconPopupButton:Show();
		else
			MacroBrokerGUI_IconPopupIcon:SetTexture("");
			MacroBrokerGUI_IconPopupButton:Hide();
		end
		if ( (selectedIcon and index == selectedIcon) or selectedIconTexture == texture ) then
			MacroBrokerGUI_IconPopupButton:SetChecked(1);
		else
			MacroBrokerGUI_IconPopupButton:SetChecked(nil);
		end
	end

	-- Scrollbar stuff
	FauxScrollFrame_Update(MacroBrokerGUI_IconPopupScrollFrame, ceil(numMacroIcons / NUM_ICONS_PER_ROW) , NUM_ICON_ROWS, MACROBROKERGUI_ICON_ROW_HEIGHT );
end

function MacroBrokerGUI.IconPopupFrame_CancelEdit()
	MacroBrokerGUI_IconPopupFrame:Hide();
	MacroBrokerGUI.Update();
end

function MacroBrokerGUI.IconPopupOkayButton_Update()
	if ( strlen(MacroBrokerGUI_IconPopupEditBox:GetText()) > 0 and
	     (MacroBrokerGUI_IconPopupFrame.mode == "edit" or MacroBrokerGUI_IconPopupFrame.selectedIcon) ) then
		MacroBrokerGUI_IconPopupOkayButton:Enable();
	else
		MacroBrokerGUI_IconPopupOkayButton:Disable();
	end
end

function MacroBrokerGUI.IconPopupButton_SelectTexture(selectedIcon)
	MacroBrokerGUI_IconPopupFrame.selectedIcon = selectedIcon;
	-- Clear out selected texture
	MacroBrokerGUI_IconPopupFrame.selectedIconTexture = nil;
	MacroBrokerGUIFrameSelectedMacroButtonIcon:SetTexture(GetMacroIconInfo(selectedIcon));
	MacroBrokerGUI.IconPopupOkayButton_Update();
	local mode = MacroBrokerGUI_IconPopupFrame.mode;
	MacroBrokerGUI_IconPopupFrame.mode = nil;
	MacroBrokerGUI.IconPopupFrame_Update(MacroBrokerGUI_IconPopupFrame);
	MacroBrokerGUI_IconPopupFrame.mode = mode;
end

function MacroBrokerGUI.IconPopupButton_OnClick(self, button)
	MacroBrokerGUI.IconPopupButton_SelectTexture(self:GetID() + (FauxScrollFrame_GetOffset(MacroBrokerGUI_IconPopupScrollFrame) * NUM_ICONS_PER_ROW))
end

function MacroBrokerGUI.IconPopupOkayButton_OnClick(self, button)
	local index = 1
	local t = MacroBrokerGUI_IconPopupLauncherCheckBox:GetChecked() and "launcher" or "data source"
	if ( MacroBrokerGUI_IconPopupFrame.mode == "new" ) then
		local name = MacroBrokerGUI_IconPopupEditBox:GetText()
		local obj
		if (not MacroBroker.IsRealNameUsed(name)) then
			local icon = MacroBrokerGUI_IconPopupFrame.selectedIcon
			icon = icon == 1 and "?" or GetMacroIconInfo(icon)
			obj = MacroBroker.CreatePlugin(name, t, nil, nil, icon, nil, nil, nil, true)
		end
		if (not obj) then
			UIErrorsFrame:AddMessage(L.OBJNAMEUSED:format(name), 1.0, 0.1, 0.1, 1.0)
			PlaySound("igQuestFailed")
			return;
		end
		MacroBroker.SetOnClickMacro(obj, "")
		if (MacroBrokerGUIFrame.macroBase > 0) then
			MacroBroker.SetCharacterSpecific(obj, true)
		end
		UpdateMacroList()
		index = MacroBrokerGUIFrame.macroBase + 1
		local max = MacroBrokerGUIFrame.macroBase == 0 and numAccountMacros or index + numCharacterMacros
		for i = index, max do
			if (MACROS[i] == name) then
				index = i
				break;
			end
		end
	elseif ( MacroBrokerGUI_IconPopupFrame.mode == "edit" ) then
		local newindex = MacroBrokerGUIFrame.selectedMacro
		local name, realname = MACROS[newindex], MacroBrokerGUI_IconPopupEditBox:GetText()
		if (MacroBroker.IsRealNameUsed(realname, name)) then
			UIErrorsFrame:AddMessage(L.OBJNAMEUSED:format(realname), 1.0, 0.1, 0.1, 1.0)
			PlaySound("igQuestFailed")
			return;
		end
		index = newindex
		local icon = MacroBrokerGUI_IconPopupFrame.selectedIcon
		icon = icon and (icon == 1 and "?" or GetMacroIconInfo(icon))
		MacroBroker.EditPlugin(name, t, nil, nil, icon)
		MacroBroker.SetRealName(name, realname)
	end
	MacroBrokerGUI_IconPopupFrame:Hide();
	MacroBrokerGUI.SelectMacro(index);
	MacroBrokerGUI.Update();
end

function MacroBrokerGUI.SaveMacro()
	if ( MacroBrokerGUIFrame.textChanged and MacroBrokerGUIFrame.selectedMacro ) then
		MacroBroker.SetOnClickMacro(MACROS[MacroBrokerGUIFrame.selectedMacro], MacroBrokerGUIFrameText:GetText())
		MacroBrokerGUIFrame.textChanged = nil;
	end
end

function MacroBrokerGUI.ToggleDeleteMacro(macro)
	if (type(macro) == "number") then  macro = MACROS[macro];  end
	if (macro) then
		if (MacroBroker.IsPluginSaved(macro)) then
			MacroBroker.RemovePlugin(macro)
		else
			MacroBroker.SavePlugin(macro)
		end
		MacroBrokerGUI.Update();
		ClearEditBoxFocus()
	end
end

function MacroBrokerGUI.LDBFieldEditBoxChanged(self, field)
	if (MacroBrokerGUIFrame.selectedMacro) then
		local text = self:GetText()
		if (text == "") then  text = nil;  end
		local name = MACROS[MacroBrokerGUIFrame.selectedMacro]
		MacroBroker.EditPluginField(name, field, text)
	end
end

function MacroBrokerGUI.EditBox_HandleTabbing(self)
	local list, index = MacroBrokerGUIFrame.EditBoxList
	for i, editbox in ipairs(list) do  -- Find current editbox's position in the list:
		if (editbox == self) then
			index = i
			break
		end
	end
	if (not index) then  return;  end  -- Editbox not in the list.

	local size, adjust = #list, IsShiftKeyDown() and -1 or 1
	local target
	repeat
		index = index + adjust
		if (index < 1) then
			index = size
		elseif (index > size) then
			index = 1
		end
		target = list[index]
	until (target:IsVisible() or target == self)

	target:SetFocus()
	return target
end


-- MACRO CONTENTS CHECK
-------------------------

local function FindSecureLine(...)
	local line, slash
	for i = 1, select("#", ...) do
		line = select(i, ...)
		slash = strmatch(line, "^(/[^%s]+)")
		if (slash and TjRunSlash.IsSecureCmd(slash, strsub(line, slash:len() + 2)) and strlower(slash) ~= "/stopmacro") then
		-- /stopmacro is allowed because we do special handling for it
			return slash, i
		end
		-- Unused: Was for returning position instead of line num to allow for highlighting; deemed unneeded.
		--pos = pos + strlen(line) + 1 -- add 1 for "\n"
	end
end

local function GetCursorLine(pos, ...)
	local len, line = 0
	for i = 1, select("#", ...) do
		line = select(i, ...)
		len = len + strlen(line) + 1 -- add 1 for "\n"
		if (pos < len) then  return line;  end
	end
end

local LastBody, SecureSlash, UnknownMacroOption

local function filterUnknownMacroOptions(self, event, arg1)
	local search = string.gsub(ERR_UNKNOWN_MACRO_OPTION_S, "%%s", "(.+)")
	local _, _, con = string.find(arg1, search)
	if (con) then
		MacroBrokerGUIFrameCharLimitText:SetText(arg1)
		MacroBrokerGUIFrameCharLimitText:SetTextColor(1, 0.1, 0.1)
		UnknownMacroOption = true
		return true
	end
end

function MacroBrokerGUI.OnTextChanged(self)
	MacroBrokerGUIFrame.textChanged = 1;
	if ( MacroBrokerGUI_IconPopupFrame.mode == "new" ) then
		MacroBrokerGUI_IconPopupFrame:Hide();
	end
	ScrollingEdit_OnTextChanged(self, self:GetParent());

	local body = self:GetText()
	if (body ~= LastBody) then
		UnknownMacroOption = nil
		local line
		ChatFrame_AddMessageEventFilter("CHAT_MSG_SYSTEM", filterUnknownMacroOptions)
		SecureSlash, line = FindSecureLine( strsplit("\n", body) )
		ChatFrame_RemoveMessageEventFilter("CHAT_MSG_SYSTEM", filterUnknownMacroOptions)
		LastBody = body
		if (SecureSlash) then
			MacroBrokerGUIFrameCharLimitText:SetFormattedText(L.LABEL_SECURECMD_ON_LINE, line, strupper(SecureSlash));
			MacroBrokerGUIFrameCharLimitText:SetTextColor(1, 0.1, 0.1)
		end
	end
end

function MacroBrokerGUI.OnCursorChanged(self)
	local body = self:GetText()
	if (body ~= LastBody or (not SecureSlash and not UnknownMacroOption)) then
		local line = GetCursorLine(self:GetCursorPosition(), strsplit("\n", body))
		local len = strlen(line)
		MacroBrokerGUIFrameCharLimitText:SetFormattedText(L.LABEL_CHAR_LIMIT_PER_LINE, len, MAX_CHARACTERS_PER_LINE);
		if (len <= MAX_CHARACTERS_PER_LINE) then
			MacroBrokerGUIFrameCharLimitText:SetTextColor(1, 1, 1)
		else
			MacroBrokerGUIFrameCharLimitText:SetTextColor(1, 0.1, 0.1)
		end
	end
end
